package scenariodiagrameditor.features;

import org.eclipse.emf.common.util.EList;
import org.eclipse.graphiti.features.IAddFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IAddConnectionContext;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.impl.AbstractAddFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Polyline;
import org.eclipse.graphiti.mm.algorithms.Rectangle;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.algorithms.styles.Orientation;
import org.eclipse.graphiti.mm.pictograms.Anchor;
import org.eclipse.graphiti.mm.pictograms.Connection;
import org.eclipse.graphiti.mm.pictograms.ConnectionDecorator;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.util.IColorConstant;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioLink;
import de.uks.se.scenariodiagram.ScenarioObject;

public class AddScenarioLinkConnectionFeature extends AbstractAddFeature implements IAddFeature
{
	private final String DEFAULT_TAG = "scenarioLinkName";

	private final int DEFAULT_LINKNAME_WIDTH = 60;
	private final int DEFAULT_LINKNAME_HEIGHT = 20;

	public AddScenarioLinkConnectionFeature(IFeatureProvider fp)
	{
		super(fp);
	}

	@Override
	public boolean canAdd(IAddContext context)
	{
		return context instanceof IAddConnectionContext && context.getNewObject() instanceof ScenarioLink;
	}

	@Override
	public PictogramElement add(IAddContext context)
	{
		IAddConnectionContext addConContext = (IAddConnectionContext) context;
		
		IPeCreateService peCreateService = Graphiti.getPeCreateService();
		IGaService gaService = Graphiti.getGaService();
		
		ScenarioLink scenarioLink = (ScenarioLink) context.getNewObject();

		//if source and target are not parent of each other
		ScenarioObject sourceObject = scenarioLink.getSource();
		ScenarioObject targetObject = scenarioLink.getTarget();
		Anchor sourceAnchor = addConContext.getSourceAnchor();
		Anchor targetAnchor = addConContext.getTargetAnchor();
		
		if(sourceObject != targetObject.getParent() &&
				targetObject != sourceObject.getParent())
		{
			// create new connection and set anchors
			Connection connection = peCreateService.createFreeFormConnection(getDiagram());
			connection.setStart(sourceAnchor);
			connection.setEnd(targetAnchor);

			// create polyline as representation of the connection
			Polyline polyline = gaService.createPlainPolyline(connection);
			polyline.setForeground(manageColor(IColorConstant.BLACK));
			polyline.setLineWidth(3);

			link(connection, scenarioLink);

			ConnectionDecorator textDecorator = peCreateService.createConnectionDecorator(connection, true, 0.5, true);
			Text text = gaService.createDefaultText(getDiagram(),textDecorator);
			text.setForeground(manageColor(IColorConstant.BLACK));
			gaService.setLocation(text, 10, 0);

			//linkname
			String value = "";
			EList<ScenarioAttribute> attributes = scenarioLink.getAttributes();
			for(ScenarioAttribute tmpAttribute : attributes){        	
				if (DEFAULT_TAG.equals(tmpAttribute.getTag()))
				{
					value = tmpAttribute.getValue();
					break;
				}
			}
			text.setValue(value);

			link(textDecorator, scenarioLink);

			return connection;
		}
		//source is parent of target or vice versa
		else
		{
			ScenarioObject childObject = null;
			ContainerShape childContainer = null;
			int linkCount = 0;

			//target is child
			if(sourceObject == targetObject.getParent())
			{
				childObject = targetObject;

				if(sourceAnchor.getParent().getLink().getBusinessObjects().contains(targetObject) && targetAnchor.getParent() instanceof ContainerShape)
				{
					childContainer = (ContainerShape) sourceAnchor.getParent();
				}
				else if(sourceAnchor.getParent() instanceof ContainerShape)
				{
					childContainer = (ContainerShape) targetAnchor.getParent();
				}
			}
			//source is child
			else
			{
				childObject = sourceObject;
				
				if(targetAnchor.getParent().getLink().getBusinessObjects().contains(sourceObject) && sourceAnchor.getParent() instanceof ContainerShape)
				{
					childContainer = (ContainerShape) targetAnchor.getParent();
				}
				else if(sourceAnchor.getParent() instanceof ContainerShape)
				{
					childContainer = (ContainerShape) sourceAnchor.getParent();
				}
			}
			
			//post-it for link to parent
			ContainerShape linkContainerShape = peCreateService.createContainerShape(childContainer, true);
			
			// rectangle
			Rectangle postIt = gaService.createRectangle(linkContainerShape);
			linkCount = childObject.getObjectType().getEReferences().size();
			int postItX = childContainer.getGraphicsAlgorithm().getWidth() - DEFAULT_LINKNAME_WIDTH;
			int postItY = childContainer.getGraphicsAlgorithm().getHeight() - DEFAULT_LINKNAME_HEIGHT + DEFAULT_LINKNAME_HEIGHT*linkCount/2;
			gaService.setLocationAndSize(postIt, postItX, postItY, DEFAULT_LINKNAME_WIDTH, DEFAULT_LINKNAME_HEIGHT);
			postIt.setFilled(false);
			postIt.setLineWidth(3);

			// text
			Shape textShape = peCreateService.createShape(linkContainerShape, false);

			//there is only one attribute
			String attributeText = scenarioLink.getAttributes().get(0).getValue();
			
			Text text = gaService.createText(textShape, attributeText);
			text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
			text.setVerticalAlignment(Orientation.ALIGNMENT_CENTER);
			gaService.setLocationAndSize(text, 0, 0, postIt.getWidth(), postIt.getHeight());
			
			resizeTargetShape(childObject, childContainer);

			peCreateService.createChopboxAnchor(linkContainerShape);
			link(textShape, scenarioLink);
			
			return childContainer;
		}
	}

	private void resizeTargetShape(ScenarioObject scenarioObject,
			ContainerShape targetContainer) {
		GraphicsAlgorithm targetShape = targetContainer.getGraphicsAlgorithm();
		int maxWidth = targetShape.getWidth();
		int maxHeight = targetShape.getHeight();

		for (Shape tmpShape : targetContainer.getChildren())
		{
			if(tmpShape instanceof ContainerShape && ((ContainerShape)tmpShape).getLink() != null &&
			  ((ContainerShape)tmpShape).getLink().getBusinessObjects().contains(scenarioObject.getParent()))
			{
				continue;
			}
			else
			{
				int xSpace = tmpShape.getGraphicsAlgorithm().getX()+tmpShape.getGraphicsAlgorithm().getWidth();
				if(maxWidth < xSpace)
				{
					maxWidth = xSpace;
				}
				int ySpace = tmpShape.getGraphicsAlgorithm().getY()+tmpShape.getGraphicsAlgorithm().getHeight();
				if(maxHeight < ySpace)
				{
					maxHeight = ySpace;
				}
			}
		}

		targetShape.setWidth(maxWidth);
		targetShape.setHeight(maxHeight);
	}
}
