package scenariodiagrameditor.features;

import org.eclipse.emf.common.util.EList;
import org.eclipse.graphiti.features.IAddFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IAddContext;
import org.eclipse.graphiti.features.impl.AbstractAddFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Polyline;
import org.eclipse.graphiti.mm.algorithms.Rectangle;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.algorithms.styles.Orientation;
import org.eclipse.graphiti.mm.pictograms.ContainerShape;
import org.eclipse.graphiti.mm.pictograms.Diagram;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;
import org.eclipse.graphiti.services.Graphiti;
import org.eclipse.graphiti.services.IGaService;
import org.eclipse.graphiti.services.IPeCreateService;
import org.eclipse.graphiti.util.IColorConstant;

import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioLink;
import de.uks.se.scenariodiagram.ScenarioObject;

public class AddScenarioObjectFeature extends AbstractAddFeature implements IAddFeature
{
	private static final IColorConstant OBJECT_FOREGROUND = IColorConstant.BLACK;
	private static final String OBJECT_TEXT_ATTRIBUTE = "scenarioObjectText";

	private static final int DEFAULT_WIDTH = 180;
	private static final int DEFAULT_HEIGHT = 60;

	private final int DEFAULT_LINKNAME_HEIGHT = 20;
	private final int DEFAULT_LINKNAME_WIDTH = 60;

	public AddScenarioObjectFeature(IFeatureProvider fp) {
		super(fp);
	}

	@Override
	public boolean canAdd(IAddContext context) {
		return context.getNewObject() instanceof ScenarioObject;
	}

	@Override
	public PictogramElement add(IAddContext context) {
		IPeCreateService peCreateService = Graphiti.getPeCreateService();
		IGaService gaService = Graphiti.getGaService();


		ScenarioObject scenarioObject = (ScenarioObject) context.getNewObject();
		ContainerShape targetContainer = context.getTargetContainer();

		ContainerShape containerShape = peCreateService.createContainerShape(targetContainer, true);
		peCreateService.createChopboxAnchor(containerShape);
		link(containerShape, scenarioObject);

		Rectangle rectangle = gaService.createRectangle(containerShape);
		gaService.setLocationAndSize(rectangle, context.getX(), context.getY(), DEFAULT_WIDTH, DEFAULT_HEIGHT);
		rectangle.setFilled(false);
		rectangle.setLineWidth(3);

		//line
		{
			Shape lineShape = peCreateService.createShape(containerShape, false);
			// create and set graphics algorithm
			Polyline polyline = gaService.createPolyline(lineShape, new int[] { 0, 20, DEFAULT_WIDTH, 20});
			polyline.setForeground(manageColor(OBJECT_FOREGROUND));
			polyline.setLineWidth(3);
		}

		//text
		{
			Shape textShape = peCreateService.createShape(containerShape, false);

			//object text
			String value = "";
			EList<ScenarioAttribute> attributes = scenarioObject.getAttributes();
			for(ScenarioAttribute tmpAttribute : attributes){        	
				if (OBJECT_TEXT_ATTRIBUTE.equals(tmpAttribute.getTag()))
				{
					value = tmpAttribute.getValue();
					break;
				}
			}

			Text text = gaService.createText(textShape, value);
			text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
			text.setVerticalAlignment(Orientation.ALIGNMENT_TOP);
			gaService.setLocationAndSize(text, 0, 0, DEFAULT_WIDTH, 20);

			link(textShape, scenarioObject);
		}


		//inserted into object
		if(!(targetContainer instanceof Diagram))
		{
			//post-it for link to parent
			{
				ContainerShape linkContainerShape = peCreateService.createContainerShape(containerShape, true);
				// rectangle
				Rectangle postIt = gaService.createRectangle(linkContainerShape);
				int postItX = containerShape.getGraphicsAlgorithm().getWidth() - DEFAULT_LINKNAME_WIDTH;
				int postItY = containerShape.getGraphicsAlgorithm().getHeight() - DEFAULT_LINKNAME_HEIGHT;
				gaService.setLocationAndSize(postIt, postItX, postItY, DEFAULT_LINKNAME_WIDTH, DEFAULT_LINKNAME_HEIGHT);
				postIt.setFilled(false);
				postIt.setLineWidth(3);

				//there is only one link
				ScenarioLink parentLink = scenarioObject.getOutgoingLinks().get(0);
				ScenarioAttribute linkName = parentLink.getAttributes().get(0);

				// text
				Shape textShape = peCreateService.createShape(linkContainerShape, false);

				String attributeText = linkName.getValue();
				Text text = gaService.createText(textShape, attributeText);
				text.setHorizontalAlignment(Orientation.ALIGNMENT_CENTER);
				text.setVerticalAlignment(Orientation.ALIGNMENT_CENTER);
				gaService.setLocationAndSize(text, 0, 0, postIt.getWidth(), postIt.getHeight());

				peCreateService.createChopboxAnchor(linkContainerShape);
				link(textShape, parentLink);
			}
			
			//resize targetShape
			resizeTargetShape(scenarioObject, targetContainer);
		}

		return containerShape;
	}

	private void resizeTargetShape(ScenarioObject scenarioObject,
			ContainerShape targetContainer) {
		GraphicsAlgorithm targetShape = targetContainer.getGraphicsAlgorithm();
		int maxWidth = targetShape.getWidth();
		int maxHeight = targetShape.getHeight();

		for (Shape tmpShape : targetContainer.getChildren())
		{
			if(tmpShape instanceof ContainerShape && ((ContainerShape)tmpShape).getLink() != null &&
			  ((ContainerShape)tmpShape).getLink().getBusinessObjects().contains(scenarioObject.getParent()))
			{
				continue;
			}
			else
			{
				int xSpace = tmpShape.getGraphicsAlgorithm().getX()+tmpShape.getGraphicsAlgorithm().getWidth();
				if(maxWidth < xSpace)
				{
					maxWidth = xSpace;
				}
				int ySpace = tmpShape.getGraphicsAlgorithm().getY()+tmpShape.getGraphicsAlgorithm().getHeight();
				if(maxHeight < ySpace)
				{
					maxHeight = ySpace;
				}
			}
		}

		targetShape.setWidth(maxWidth);
		targetShape.setHeight(maxHeight);
	}
}
