package scenariodiagrameditor.features;

import java.util.HashSet;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.IDirectEditingContext;
import org.eclipse.graphiti.features.impl.AbstractDirectEditingFeature;
import org.eclipse.graphiti.mm.algorithms.GraphicsAlgorithm;
import org.eclipse.graphiti.mm.algorithms.Text;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;
import org.eclipse.graphiti.mm.pictograms.Shape;

import scenariodiagrameditor.util.Util;
import de.uks.se.scenariodiagram.ScenarioAttribute;
import de.uks.se.scenariodiagram.ScenarioDiagram;
import de.uks.se.scenariodiagram.ScenarioObject;

public class DirectEditScenarioObjectFeature extends AbstractDirectEditingFeature
{
	private final String DEFAULT_TAG = "scenarioObjectText";
	private final String DEFAULT_OBJECT_TYPE = "DefaultType";

	public DirectEditScenarioObjectFeature(IFeatureProvider fp)
	{
		super(fp);
	}

	@Override
	public int getEditingType()
	{
		return TYPE_TEXT;
	}

	@Override
	public String getInitialValue(IDirectEditingContext context)
	{
		PictogramElement pe = context.getPictogramElement();
		ScenarioObject scenarioObject = (ScenarioObject) getBusinessObjectForPictogramElement(pe);
		String value = "";

		EList<ScenarioAttribute> attributes = scenarioObject.getAttributes();
		for(ScenarioAttribute tmpAttribute : attributes)
		{        	
			if (DEFAULT_TAG.equals(tmpAttribute.getTag()))
			{
				value = tmpAttribute.getValue();
				break;
			}
		}

		return value;
	}

	@Override
	public boolean canDirectEdit(IDirectEditingContext context)
	{
		PictogramElement pe = context.getPictogramElement();
		Object object = getBusinessObjectForPictogramElement(pe);
		GraphicsAlgorithm ga = context.getGraphicsAlgorithm();

		if (object instanceof ScenarioObject && ga instanceof Text)
		{
			return true;
		}

		return false;
	}

	@Override
	public void setValue(String value, IDirectEditingContext context) {
		PictogramElement pe = context.getPictogramElement();
		ScenarioObject scenarioObject = (ScenarioObject) getBusinessObjectForPictogramElement(pe);

		if(scenarioObject != null)
		{
			updateModels(value, scenarioObject);

			Text text = (Text)context.getGraphicsAlgorithm();
			text.setValue(value);

			updatePictogramElement(((Shape) pe).getContainer());

			getDiagramBehavior().refreshContent();
		}
	}

	private void updateModels(String value, ScenarioObject scenarioObject)
	{
		//textAttribute is always first
		ScenarioAttribute textAttribute = scenarioObject.getAttributes().get(0);
		textAttribute.setValue(value);

		//should always have an old type
		if(scenarioObject.getObjectType() != null)
		{
			//always set type to null
			scenarioObject.setObjectType(null);

			//is typename defined
			if(value.contains(":") && value.split(":").length > 1)
			{
				String typeName = value.split(":")[1];

				ScenarioDiagram scenarioDiagram = (ScenarioDiagram) getBusinessObjectForPictogramElement(getDiagram());
				for(EClass tmpType : scenarioDiagram.getObjectTypes())
				{
					// existing type
					if(tmpType.getInstanceTypeName().equals(typeName))
					{
						scenarioObject.setObjectType(tmpType);
						break;
					}
				}

				// new type
				if(scenarioObject.getObjectType() == null)
				{	
					EClass newScenarioObjectType = EcoreFactory.eINSTANCE.createEClass();
					newScenarioObjectType.setInstanceTypeName(typeName);
					scenarioDiagram.getObjectTypes().add(newScenarioObjectType);
					scenarioObject.setObjectType(newScenarioObjectType);

					EAttribute newAttributeType = EcoreFactory.eINSTANCE.createEAttribute();
					newScenarioObjectType.getEStructuralFeatures().add(newAttributeType); //frs Speichern
					textAttribute.setAttributeType(newAttributeType);
					newAttributeType.setName(DEFAULT_TAG);
					newAttributeType.setEType(Util.getType(""));
					
					//copy attributes
					copyAttributes(scenarioObject, newScenarioObjectType);
					
					//copy objects and links
				}
				//existing type
				else
				{
					EClass existingType = scenarioObject.getObjectType();
					
					//copy attributes
					copyAttributes(scenarioObject, existingType);
					
					// objects and links
				}
			}
			//default object type
			else
			{
				ScenarioDiagram scenarioDiagram = (ScenarioDiagram) getBusinessObjectForPictogramElement(getDiagram());
				for(EClass tmpType : scenarioDiagram.getObjectTypes())
				{
					// default object type
					if(tmpType.getInstanceTypeName().equals(DEFAULT_OBJECT_TYPE))
					{
						scenarioObject.setObjectType(tmpType);
						
						//copy attributes
						copyAttributes(scenarioObject, tmpType);
						
						//objects and links
					}
				}
			}
		}
	}

	private void copyAttributes(ScenarioObject scenarioObject, EClass existingType)
	{
		for(ScenarioAttribute attribute : scenarioObject.getAttributes())
		{
			//make a copy?
			if(existingType.getEStructuralFeature(attribute.getTag()) == null)
			{
				EAttribute copyType = attribute.getAttributeType();
				
				EAttribute copyAttributeType = EcoreFactory.eINSTANCE.createEAttribute();
				existingType.getEStructuralFeatures().add(copyAttributeType); //frs Speichern
				attribute.setAttributeType(copyAttributeType);
				copyAttributeType.setName(copyType.getName());
				copyAttributeType.setEType(copyType.getEType());						
			}
		}
	}

	@Override
	public boolean isAutoCompletionEnabled()
	{
		return true;
	}

	@Override
	public boolean isCompletionAvailable()
	{
		return true;
	}

	@Override
	public String[] getValueProposals(String value, int caretPos, IDirectEditingContext context)
	{
		String typeName = "";

		//no type expected
		if(!value.contains(":"))
		{
			return new String[0];
		}

		//type expected
		String[] tagAndValue = value.split(":");
		if(tagAndValue.length > 1)
		{
			//given part of typename
			typeName = tagAndValue[1];
		}

		//search for matching type names
		ScenarioDiagram scenarioDiagram = (ScenarioDiagram) getBusinessObjectForPictogramElement(getDiagram());
		EList<EClass> objectTypes = scenarioDiagram.getObjectTypes();

		HashSet<String> proposals = new HashSet<String>();

		for (EClass tmpClass : objectTypes) {
			String instanceTypeName = tmpClass.getInstanceTypeName();
			//is there an objectName and (show all typenames or matching typenames)
			if(tagAndValue.length > 0 && (typeName.equals("") || instanceTypeName.startsWith(typeName)))
			{
				proposals.add(tagAndValue[0] + ":" + instanceTypeName);
			}
		}

		String[] result = proposals.toArray(new String[proposals.size()]);

		return result;
	}
}
